#
# Flatpak installation spoke
#
# Copyright (C) 2023 Your Name
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from pyanaconda.anaconda_loggers import get_module_logger
from pyanaconda.core.i18n import _
from pyanaconda.core.configuration.anaconda import conf
from pyanaconda.ui.gui.spokes import StandaloneSpoke
from pyanaconda.ui.gui.utils import gtk_action_wait
from pyanaconda import flags
from pyanaconda.core.i18n import _
from pyanaconda.core.configuration.anaconda import conf
from pyanaconda.ui.gui.spokes.standalone import StandaloneSpoke
from pyanaconda.ui.categories.software import SoftwareCategory
from pyanaconda.ui.lib.software import get_software_support_addon_paths
from pyanaconda.ui.gui.utils import escape_markup, set_password_visibility
import os
import gi

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk

__all__ = ["FlatpakSpoke"]

log = get_module_logger(__name__)

class FlatpakSpoke(StandaloneSpoke):
    """Spoke for Flatpak installation.

    This class provides a way to enable Flatpak installation during the
    installation process.
    """
    
    builderObjects = ["flatpakSpoke"]
    mainWidgetName = "flatpakSpoke"
    uiFile = "flatpak.glade"
    helpFile = "FlatpakSpoke.xml"
    
    # StandaloneSpoke attributes
    preForHub = None  # Show before the main hub
    postForHub = "SummaryHub"  # Show after the summary hub
    priority = 10  # Run after other post-installation steps

    def __init__(self, data, storage, payload):
        super().__init__(storage, payload)
        self._flatpak_enabled = False
        self._flatpak_remote = "flathub"
        self._flatpak_remote_url = "https://dl.flathub.org/repo/flathub.flatpakrepo"
    
    @classmethod
    def should_run(cls, environment, data):
        """Should this spoke be shown?
        
        This method is called to determine if the spoke should be shown.
        We want to show it in all environments except for automated installs.
        """
        if environment == "anaconda" and flags.automatedInstall:
            return False
        return True
        
    def initialize(self):
        StandaloneSpoke.initialize(self)
        
        # Get the main checkbox
        self._flatpak_checkbox = self.builder.get_object("flatpak_checkbox")
        self._flatpak_checkbox.connect("toggled", self.on_flatpak_toggled)
        
        # Set initial state
        self._flatpak_checkbox.set_active(False)
    
    def refresh(self):
        """Refresh the UI elements."""
        # Set the checkbox state
        self._flatpak_checkbox.set_active(self._flatpak_enabled)
    
    @property
    def completed(self):
        """Has this spoke been completed?"""
        # Always consider it completed, as it's optional
        return True
        
    @property
    def mandatory(self):
        """Is this spoke mandatory?"""
        return False
        
    @property
    def status(self):
        """Return a brief status string"""
        if self._flatpak_enabled:
            return _("Flatpak will be configured")
        return _("Flatpak will not be configured")
        
    def apply(self):
        """Apply the Flatpak configuration"""
        self._flatpak_enabled = self._flatpak_checkbox.get_active()
        if self._flatpak_enabled:
            conf.payload.flatpak_remote = (self._flatpak_remote, self._flatpak_remote_url)
        else:
            conf.payload.flatpak_remote = None
    
    @property
    def ready(self):
        """Is the spoke ready to continue?"""
        return True
    
    @property
    def completed(self):
        """Is the spoke completed?"""
        # This spoke is always completed as it's optional
        return True
    
    @property
    def status(self):
        """Return a status summary."""
        if self._flatpak_enabled:
            return _("Flatpak applications will be installed")
        return _("Flatpak applications will not be installed")
    
    @property
    def mandatory(self):
        """Is the spoke mandatory?"""
        return False
    
    def on_flatpak_toggled(self, button):
        """Handle Flatpak checkbox toggling."""
        self._flatpak_enabled = button.get_active()
        self.window.emit("button-clicked", {})
    
    def on_back_clicked(self, button):
        """Handle back button click."""
        self.apply()
        StandaloneSpoke.on_back_clicked(self, button)
    
    def on_next_clicked(self, button):
        """Handle next button click."""
        self.apply()
        StandaloneSpoke.on_next_clicked(self, button)
